
/*
 * $Id: libnet.i,v 1.1.1.1 2001/10/15 06:54:54 davidma Exp $
 *
 * Python libnet
 * Copyright (C) 2001, David Margrave (davidma@eskimo.com)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */


%module libnet


%{
#include <libnet.h>
#include "constants.h"
#include "pylibnet.h"
%}

%include constants.i

typedef unsigned short u_short;
typedef unsigned long u_long;
typedef unsigned char u_char;


%init %{
  /* d is the dictionary for the current module */
  init_errors(d);

%}

%exception {
  int err;
  clear_exception();
  PyErr_Clear();
  $action
  if ((err = check_exception())) {
    set_error(err, get_exception_message());
    return NULL;
  }
  else if(PyErr_Occurred()) {
    return NULL;
  }
}

/* typemaps */
%typemap(in) u_char *{
  if (!($1=PyString_AsString($input))) {
    PyErr_SetString(PyExc_TypeError,"expected a string");
    return NULL;
  }
}

%typemap(out) u_char *{
  $result=PyString_FromString($1);
}

%typemap(in) u_long {
  $1=PyLong_AsUnsignedLong($input);
}

%typemap(out) u_long {
  $result=PyLong_FromUnsignedLong($1);
}


/* let functions return raw python objects */
%typemap(out) PyObject * {
  $result = $1;
}

/* let functions take raw python objects */
%typemap(in) PyObject * {
  $1 = $input;
}


typedef struct {
  struct libnet_link_int *link;
  int fd;
  char *device;
  %extend {
    interface();
    ~interface();
    void open_link(char *device);
    void open_raw(int protocol);
    %feature("shadow") write(PyObject *) %{
    def write(self, arg0):
      if isinstance(arg0, packet):
        arg0 = repr(arg0)
        return $action(self, arg0)
      raise TypeError, "expected type packet"
    %}
    void write (PyObject *pkt);
    u_long get_ipaddr();
    PyObject *get_hwaddr();
  }
} interface;

/* packet builder routines */
PyObject *build_arp(u_short hrd, u_short pro, u_char hln, u_char pln,
                    u_short op, u_char *sha, u_char *spa, u_char *tha,
                    u_char *tpa);
PyObject *build_dns(u_short id, u_short flags, u_short num_q,
                    u_short num_anws_rr, u_short num_auth_rr,
                    u_short num_addi_rr);
PyObject *build_ethernet(u_char *dst, u_char *src, u_short type);

PyObject *build_icmp_echo(u_char type, u_char code, u_short id, u_short seq);
PyObject *build_icmp_mask(u_char type, u_char code, u_short id, u_short seq,
                         u_long mask);
PyObject *build_icmp_unreach(u_char type, u_char code, u_short orig_len,
        u_char orig_tos, u_short orig_id, u_short orig_frag, u_char orig_ttl,
        u_char orig_prot, u_long orig_src, u_long orig_dst, PyObject *payload);
PyObject *build_icmp_timeexceed(u_char type, u_char code, u_short orig_len,
        u_char orig_tos, u_short orig_id, u_short orig_frag, u_char orig_ttl,
        u_char orig_prot, u_long orig_src, u_long orig_dst, PyObject *payload);
PyObject *build_icmp_timestamp(u_char type,u_char code,u_short id,u_short seq,
        n_time otime, n_time rtime, n_time ttime);
PyObject *build_icmp_redirect(u_char type, u_char code, u_long gateway,
            u_short orig_len, u_char orig_tos, u_short orig_id,
            u_short orig_frag, u_char orig_ttl, u_char orig_prot,
            u_long orig_src, u_long orig_dst, PyObject *payload);





PyObject *build_ip(u_short len, u_char tos, u_short id, u_short frag,
                          u_char ttl, u_char prot, u_long src, u_long dst);
/* ospf etc. will be added later */
PyObject *build_tcp(u_short sp, u_short dp, u_long seq, u_long ack,
                           u_char control, u_short win, u_short urg);
PyObject *build_udp(u_short sp, u_short dp);


void do_checksum(PyObject *pkt, int protocol, int packet_size);
u_char *select_device(PyObject *device);
u_char *host_lookup(u_long in, u_short use_name);
u_long name_resolve(u_char *hostname, u_short use_name);


/* simpler python classes */

%pythoncode %{

import string

def dumphex(s):
  bytes = map(lambda x: '%.2x ' % x, map(ord, s))
  if len(s)>16:
    for i in range((len(s)+16)/16*16, 15, -16):
      bytes.insert(i,'\n')
  print string.join(bytes, '')

def gethex(s):
  bytes = map(lambda x: '%.2x ' % x, map(ord, s))
  if len(s)>16:
    for i in range((len(s)+16)/16*16, 15, -16):
      bytes.insert(i,'\n')
  return string.join(bytes, '')


protocol_headers = ['tcp_options', 'tcp_header', 'udp_header', 'icmp_header']
# some purist will jump on me and say "internetwork" here.  tough
network_headers = ['arp_header', 'ip_header', 'ip_options']
mac_headers = ['eth_header']

class packet:
  def __init__(self):
    self.payload=None
    self.protocol_headers={}
    self.network_headers={}
    self.mac_headers={}

  def do_checksum(self, protocol, pkt_size):
    val = self.getvalue(macheader=0)
    ip_hdrlen = (ord(val[0]) & 0x0f) * 4
    _libnet.do_checksum(val, protocol, pkt_size)
    if protocol==IPPROTO_TCP:
      self.protocol_headers['tcp_header'] = val[ip_hdrlen:ip_hdrlen+pkt_size]
    if protocol==IPPROTO_IP:
      self.network_headers['ip_header'] = val[0:ip_hdrlen]
    elif protocol==IPPROTO_ICMP:
      self.protocol_headers['icmp_header'] = val[ip_hdrlen:]
    if protocol==IPPROTO_TCP:
      self.protocol_headers['tcp_header'] = val[ip_hdrlen:ip_hdrlen+pkt_size]
    if protocol==IPPROTO_UDP:
      self.protocol_headers['udp_header'] = val[ip_hdrlen:ip_hdrlen+pkt_size]

  def getvalue(self, macheader=1):
    value = ''
    if macheader:
      for val in mac_headers:
        if self.mac_headers.has_key(val):
          value = self.mac_headers[val]
          break
    for val in network_headers:
      if self.network_headers.has_key(val):
        value = value + self.network_headers[val]
        break
    for val in protocol_headers:
      if self.protocol_headers.has_key(val):
        value = value + self.protocol_headers[val]
        break
    if self.payload:
      value = value + self.payload
    return value

  def __repr__(self):
    return self.getvalue()

  def __len__(self):
    return len(self.__repr__())

  def __str__(self):
    return gethex(self.__repr__())
     
     
  def build_ethernet(self, *args, **kwargs):
    self.mac_headers['eth_header'] = apply(_libnet.build_ethernet, args, kwargs)

  def build_arp(self, *args, **kwargs):
    self.network_headers['arp_header'] = apply(_libnet.build_arp, args, kwargs)

  def build_ip(self, *args, **kwargs):
    self.network_headers['ip_header'] = apply(_libnet.build_ip, args, kwargs)

  def build_icmp_echo(self, *args, **kwargs):
    self.protocol_headers['icmp_header'] = apply(_libnet.build_icmp_echo, args, kwargs)

  def build_icmp_mask(self, *args, **kwargs):
    self.protocol_headers['icmp_header'] = apply(_libnet.build_icmp_mask, args, kwargs)

  def build_icmp_unreach(self, *args, **kwargs):
    self.protocol_headers['icmp_header'] = apply(_libnet.build_icmp_unreach, args, kwargs)

  def build_icmp_timeexceeded(self, *args, **kwargs):
    self.protocol_headers['icmp_header'] = apply(_libnet.build_icmp_timeexceeded, args, kwargs)

  def build_icmp_redirect(self, *args, **kwargs):
    self.protocol_headers['icmp_header'] = apply(_libnet.build_icmp_redirect, args, kwargs)

  def build_icmp_timestamp(self, *args, **kwargs):
    self.protocol_headers['icmp_header'] = apply(_libnet.build_icmp_timestamp, args, kwargs)

  def build_tcp(self, *args, **kwargs):
    self.protocol_headers['tcp_header'] = apply(_libnet.build_tcp, args, kwargs)
    
  def build_udp(self, *args, **kwargs):
    self.protocol_headers['udp_header'] = apply(_libnet.build_udp, args, kwargs)

  def build_dns(self, *args, **kwargs):
    self.payload = apply(_libnet.build_dns, args, kwargs)

%}


